/* Small shell, illustrating fork(), exec(), wait() */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>

/* This function prints a prompt, reads in a command
   line,  and  parses it,  placing  pointers  to the
   individual  tokens in the args  array,  which  is
   terminated by a NULL pointer
*/

int prompt_and_parse(char **args)
{
    static char line[100];
    printf("> ");

    /* gets() is not safe because there is no way to
       prevent  running  over the end of the buffer.
       But we use it anyway ...

       gets() will return NULL if it reaches "end of
       file" -- i.e. if the user enters ^D
    */

    if (gets(line) == NULL)
        return -1;   /* End of input, shell will exit */

    /* Successive calls to strtok()  pull out the
       arguments from the command line one by one,
       terminating each with a null (0) character.
    */

    *args++ = strtok(line, " \t");
    while (*args++ = strtok(NULL, " \t"))
        /* Empty body */ ;
    return 1;
}

main()
{
    char *args[20];

    /* We will ignore SIGINT (generated by ctrl-C) */

    signal(SIGINT, SIG_IGN);

    /* Main command loop */
    while (prompt_and_parse(args) > 0) {
        if (fork() == 0) { /* Child */

            /* The first argument to  execvp is the file
               name of the executable. The second arg is
               an  array  of  strings  which  passes the
               "command line arguments"  of the program.
            */

	    execvp(args[0], args);

            /* Only come here if the exec fails */

	    printf("%s: not found\n", args[0]);
	    exit(1);
	}
	else { /* Parent */
	    wait(0);
	}
    }
}

