package com.example.maze;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;


// NB have to implement SensorEventListener
public class MazeActivity extends Activity implements SensorEventListener
{

  private static final int RESTART_MENU_ID = Menu.FIRST;
  private static final int NEWGAME_MENU_ID = RESTART_MENU_ID + 1;
  private static final int NANOSECS_IN_SEC = 1000000000;
  public static final String SHARED_PREFS_NAME = "MazePrefs";
  public static final String HI_SCORE = "hiscore";
  public static final String TIMETAKEN_KEY = "timetaken";
  public static final int NAVIGATE_ID = 0;

  private boolean gameOver = false;
  protected static MazeView maze;
  private SensorManager manager;
  private Sensor accel;
  private MazeBall ball;
  private long startNanoSecs;
  private long saveNanoSecs;
  private static String TAG = "MazeActivity";

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);

    ball = (MazeBall)findViewById(R.id.mazeball);
    maze = (MazeView)findViewById(R.id.maze);
    manager = (SensorManager)getSystemService(SENSOR_SERVICE);
    accel = manager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
    startGame();
  }

  public void startGame() {
    manager.registerListener(this, accel, 
        SensorManager.SENSOR_DELAY_GAME);
    startTimer();
  }

  public void newGame() {
    //gameOver = false;
    maze.newMaze();
    ball.newBall();
    startGame();
  }

  public void restartThisGame() {
    //gameOver = false;
    ball.newBall();
    startGame();
  }

  public void finishGame() {
    long timeNanoSecs = System.nanoTime();
    long timeTakenNanoSecs = (timeNanoSecs - startNanoSecs) + saveNanoSecs;

    Toast.makeText(this.getApplicationContext(), R.string.finished_message, Toast.LENGTH_SHORT).show();

    SharedPreferences prefs = getSharedPreferences(SHARED_PREFS_NAME, 0);
    long currentHiScore = prefs.getLong(HI_SCORE, 0); 
    int scoreSecs = (int) (timeTakenNanoSecs / NANOSECS_IN_SEC);
    Toast.makeText(this.getApplicationContext(), "Score: " + scoreSecs + " s", Toast.LENGTH_LONG).show();

    if ((currentHiScore == 0) || (timeTakenNanoSecs < currentHiScore)) {
      SharedPreferences.Editor prefEditor = prefs.edit();
      prefEditor.putLong(HI_SCORE, timeTakenNanoSecs);
      prefEditor.commit();
      Toast.makeText(this.getApplicationContext(), "New high score " + scoreSecs + " s", Toast.LENGTH_LONG).show();
    }
    manager.unregisterListener(this);
  }

  public void onAccuracyChanged(Sensor sensor, int accuracy) {
    ball.invalidate();
  }

  public void onSensorChanged(SensorEvent event) {
    int[] topCornerValues = ball.moveBall(event.values[0], event.values[1]);
    ball.invalidate();
    if (isBallAtEnd( topCornerValues[0], topCornerValues[1] )) {
      finishGame();
    }
    // See article text: use these lines instead of the above 3 for one way
    // of solving the perpetual-Toast problem.
    /*if ( (isBallAtEnd( topCornerValues[0], topCornerValues[1] )) && !gameOver ) {
      finishGame();
      gameOver = true;
    }*/
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    super.onCreateOptionsMenu(menu);
    menu.add(Menu.NONE, RESTART_MENU_ID, Menu.NONE, R.string.menu_restart);
    menu.add(Menu.NONE, NEWGAME_MENU_ID, Menu.NONE, R.string.menu_newgame);
    return true;
  }

  @Override
  public boolean onMenuItemSelected(int id, MenuItem item) {
    switch(item.getItemId()) {
      case RESTART_MENU_ID:
        restartThisGame();
        return true;
      case NEWGAME_MENU_ID:
        newGame();
        return true;
      default:
        // we don't have any other menu items
    }
    return super.onMenuItemSelected(id, item);
  }

  protected void onResume() {
    super.onResume();
	startTimer();
    manager.registerListener(this, accel, 
        SensorManager.SENSOR_DELAY_GAME);
  }

  protected void onPause() {
    super.onPause();
	pauseTimer();
    manager.unregisterListener(this);
  }

  private void startTimer() {
    startNanoSecs = System.nanoTime();
  }

  private void pauseTimer() {
	long pauseNanoSecs = System.nanoTime();
	saveNanoSecs = pauseNanoSecs - startNanoSecs;
  }

  private boolean isBallAtEnd(int x, int y) {
    int[] midBall = ball.cellBallMiddle(x, y);
	return maze.isLastCell(midBall[0], midBall[1]);
  }
}
