package com.example.maze;

import android.app.Activity;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.OvalShape;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

public class MazeBall extends View {
  private int diameter;                 
  private int cellWidth;
  private static float TRANSPARENCY = 0;
  private ShapeDrawable ball;
  private Context context;
  private static final String TAG = "MazeBall";

  private static int TOP_MAZE_X = 0;
  private static int TOP_MAZE_Y = 1;
  private int topX = TOP_MAZE_X;
  private int topY = TOP_MAZE_Y;

  Paint paint = new Paint();

  public MazeBall(Context context, AttributeSet attrs) {
    super(context, attrs);
	this.context = context;
    initVars(attrs);
    init();
  }

  public void newBall() {
    topX = TOP_MAZE_X;
    topY = TOP_MAZE_Y;
    ball.setBounds(topX, topY, diameter+topX, diameter+topY);
  }

  private void initVars(AttributeSet attrs) {
    TypedArray a = getContext().obtainStyledAttributes(attrs,R.styleable.Maze);
     
     cellWidth = (int)a.getDimension(R.styleable.Maze_cellWidth, 0);
     diameter  = (int)a.getDimension(R.styleable.Maze_ballDiam, 0);
     a.recycle();
     
     if (cellWidth <= 0 || diameter <= 0 ) {
       throw new RuntimeException("Valid integers for the maze dimensions must be passed in to this class via XML parameters");
     }
  }

  private void init() {
    ball = new ShapeDrawable(new OvalShape());
    ball.getPaint().setColor(Color.GREEN);
    ball.setBounds(topX, topY, diameter+topX, diameter+topY);
  }

  protected void onDraw(Canvas canvas) {
    ball.draw(canvas);
  }

  protected int[] moveBall(float x, float y) {
    int newX = topX - (int)x;
    int newY = topY + (int)y;
    boolean movable = canBallMove(newX, newY);
    if (movable) {
      topX = newX;
      topY = newY;
    } else {
     // ball stays where it is as it cannot move
    }
    ball.setBounds(topX, topY, diameter+topX, diameter+topY);
    return new int[] { topX, topY };
  }

  private boolean canBallMove(int x, int y) {
    int newMidX = x + diameter/2;
    int newMidY = y + diameter/2;

    // Find out which cell ball's middle was in.
    int[] midValues = cellBallMiddle(x, y);
    int oldCellX = midValues[0];
    int oldCellY = midValues[1];
    // Find out which cell ball's middle would be in now
    int newCellX = newMidX/cellWidth + 1;
    int newCellY = newMidY/cellWidth + 1;
    if ((newMidX % cellWidth) + diameter/2 >= cellWidth) {
      newCellX++;
    }
    if ((newMidX % cellWidth) - diameter/2 <= 0) {
      newCellX--;
    }
    if ((newMidY % cellWidth) + diameter/2 >= cellWidth) {
      newCellY++;
    }
    if ((newMidY % cellWidth) - diameter/2 <= 0) {
      newCellY--;
    }

    return !(MazeActivity.maze.isWall(oldCellX, oldCellY, newCellX, newCellY));
  }


  protected int[] cellBallMiddle(int x, int y) {
    // Note that this division will round down, which is correct
    int midX = (x + diameter/2)/cellWidth + 1;
    int midY = (y + diameter/2)/cellWidth + 1;
    return new int[] { midX, midY };
  }
}


